﻿// =============================
// == scaleCompositions.jsx ==
// =============================

/**
 * ScaleCompositions Module
 * Contient les fonctions nécessaires pour mettre à l'échelle une composition dans After Effects.
 */

var ScaleCompositions = (function() {
    return {
        /**
         * Teste si une nouvelle échelle est valide pour une composition.
         * @param {CompItem} activeItem - La composition active.
         * @param {Number} test_scale - Le facteur d'échelle à tester.
         * @returns {Boolean} - Vrai si l'échelle est valide, sinon faux.
         */
        testNewScale: function(activeItem, test_scale) {
            if (!activeItem || !(activeItem instanceof CompItem)) {
                alert("Veuillez sélectionner ou ouvrir une composition d'abord.", "Test Nouvelle Échelle");
                return false;
            }
            if (
                test_scale * activeItem.width < 1 || test_scale * activeItem.width > 30000 ||
                test_scale * activeItem.height < 1 || test_scale * activeItem.height > 30000
            ) {
                return false;
            }
            return true;
        },

        /**
         * Met à l'échelle tous les zooms des caméras dans une composition.
         * @param {CompItem} theComp - La composition à traiter.
         * @param {Number} scaleBy - Le facteur d'échelle.
         */
        scaleAllCameraZooms: function(theComp, scaleBy) {
            for (var i = 1; i <= theComp.numLayers; i++) {
                var curLayer = theComp.layer(i);
                if (curLayer.matchName === "ADBE Camera Layer") {
                    var curZoom = curLayer.zoom;
                    if (curZoom.numKeys === 0) {
                        curZoom.setValue(curZoom.value * scaleBy);
                    } else {
                        for (var j = 1; j <= curZoom.numKeys; j++) {
                            curZoom.setValueAtKey(j, curZoom.keyValue(j) * scaleBy);
                        }
                    }
                }
            }
        },

        /**
         * Définit un calque parent pour tous les calques non parentés d'une composition.
         * @param {CompItem} theComp - La composition à traiter.
         * @param {AVLayer} newParent - Le nouveau calque parent.
         */
        makeParentLayerOfAllUnparented: function(theComp, newParent) {
            for (var i = 1; i <= theComp.numLayers; i++) {
                var curLayer = theComp.layer(i);
                var wasLocked = curLayer.locked;
                curLayer.locked = false;
                if (curLayer !== newParent && curLayer.parent === null) {
                    curLayer.parent = newParent;
                }
                curLayer.locked = wasLocked;
            }
        },

        /**
         * Applique la mise à l'échelle à une composition.
         * @param {CompItem} activeComp - La composition à mettre à l'échelle.
         * @param {Number} scale_factor - Le facteur d'échelle.
         */
        scaleComposition: function(activeComp, scale_factor) {
            try {
                // Commencer le groupe d'annulation
                app.beginUndoGroup("Mise à l'échelle de la composition");

                // Créer un calque nul 3D
                var null3DLayer = activeComp.layers.addNull();
                null3DLayer.threeDLayer = true;
                null3DLayer.name = "Super Parent";
                null3DLayer.position.setValue([0, 0, 0]);

                // Définir le calque nul comme parent des calques non parentés
                this.makeParentLayerOfAllUnparented(activeComp, null3DLayer);

                // Mettre à jour la taille de la composition
                activeComp.width = Math.floor(activeComp.width * scale_factor);
                activeComp.height = Math.floor(activeComp.height * scale_factor);

                // Mettre à l'échelle les zooms des caméras
                this.scaleAllCameraZooms(activeComp, scale_factor);

                // Mettre à l'échelle le calque nul
                var superParentScale = null3DLayer.scale.value;
                superParentScale[0] *= scale_factor;
                superParentScale[1] *= scale_factor;
                superParentScale[2] *= scale_factor;
                null3DLayer.scale.setValue(superParentScale);

                // Supprimer le calque nul
                null3DLayer.remove();

                // Terminer le groupe d'annulation
                app.endUndoGroup();

                alert("La composition a été mise à l'échelle avec succès.", "Succès");
            } catch (e) {
                alert("Erreur dans scaleComposition : " + e.toString(), "Erreur");
            }
        }
    };
})();
