

    /**
     * Récupère la valeur numérique de la propriété demandée pour un calque donné.
     * @param {Layer} layer 
     * @param {String} propertyName - ex: "Opacité", "Échelle X", "Rotation X (3D)", etc.
     * @returns {Number} une valeur numérique pour le tri
     */
    function getPropertyValue(layer, propertyName) {
        // Si ce n'est pas un AVLayer (ex: caméra, lumière, etc.), on renvoie 0 par défaut
        if (!(layer instanceof AVLayer)) {
            return 0;
        }

        // Raccourcis vers transform
        var t = layer.transform;
        
        // Parfois, pour des calques texte / forme, certaines propriétés 3D n'existent pas,
        // ou layer.threeDLayer === false. On fera un check "layer.threeDLayer" au besoin.

        switch (propertyName) {
            case "Opacité":
                return t.opacity.value;

            case "Échelle (moyenne)":
                // Échelle = [x%, y%, ...], on prend la moyenne (2D)
                var sMean = (t.scale.value[0] + t.scale.value[1]) / 2;
                return sMean;

            case "Échelle X":
                return t.scale.value[0];

            case "Échelle Y":
                return t.scale.value[1];

            case "Rotation (2D)":
                // Valeur rotation 2D = transform.rotation
                return t.rotation.value;

            case "Rotation X (3D)":
                if (layer.threeDLayer) {
                    return t.xRotation.value;
                } else {
                    return 0; 
                }

            case "Rotation Y (3D)":
                if (layer.threeDLayer) {
                    return t.yRotation.value;
                } else {
                    return 0;
                }

            case "Rotation Z (3D)":
                if (layer.threeDLayer) {
                    return t.zRotation.value;
                } else {
                    // Pour un calque 2D, rotation Z = rotation classique
                    return t.rotation.value;
                }

            case "Position (distance à l'origine)":
                var p = t.position.value; // [x, y] ou [x, y, z]
                // On calcule la distance euclidienne depuis (0,0,0) par exemple
                var x = p[0], y = p[1], z = (p.length > 2) ? p[2] : 0;
                return Math.sqrt(x * x + y * y + z * z);

            case "Position X":
                return t.position.value[0];

            case "Position Y":
                return t.position.value[1];

            case "Position Z (3D)":
                if (t.position.value.length > 2) {
                    return t.position.value[2];
                } else {
                    return 0;
                }

            case "Point d'ancrage X":
                return t.anchorPoint.value[0];

            case "Point d'ancrage Y":
                return t.anchorPoint.value[1];

            case "Point d'ancrage Z (3D)":
                if (t.anchorPoint.value.length > 2) {
                    return t.anchorPoint.value[2];
                } else {
                    return 0;
                }

            case "InPoint":
                // Début du calque (en secondes)
                return layer.inPoint;

            case "OutPoint":
                // Fin du calque (en secondes)
                return layer.outPoint;

            case "Durée (Out - In)":
                return (layer.outPoint - layer.inPoint);

            default:
                // Si la propriété n'est pas reconnue, on renvoie 0
                return 0;
        }
    }

    /**
     * Trie les calques sélectionnés selon la propriété, en ordre croissant ou décroissant.
     * @param {String} propertyName  - nom de la propriété
     * @param {Boolean} ascending    - true => ordre croissant, false => ordre décroissant
     */
    function sortSelectedLayers(propertyName, ascending) {
        var comp = app.project.activeItem;
        if (!comp || !(comp instanceof CompItem)) {
            alert("Veuillez sélectionner une composition avant de lancer ce script.");
            return;
        }

        var selectedLayers = comp.selectedLayers;
        if (selectedLayers.length < 2) {
            alert("Veuillez sélectionner au moins deux calques.");
            return;
        }

        // Ouvrir un groupe d'annulations
        app.beginUndoGroup("Trier les calques par " + propertyName);

        // Construire un tableau d'objets {layer, value}
        var layerData = [];
        for (var i = 0; i < selectedLayers.length; i++) {
            var lyr = selectedLayers[i];
            var val = getPropertyValue(lyr, propertyName);
            layerData.push({
                layer: lyr,
                value: val
            });
        }

        // Trier
        layerData.sort(function (a, b) {
            return ascending ? (a.value - b.value) : (b.value - a.value);
        });

        // Réordonner les calques : on parcourt le tableau du dernier au premier
        // et on les "moveToBeginning()"
        for (var j = layerData.length - 1; j >= 0; j--) {
            layerData[j].layer.moveToBeginning();
        }

        app.endUndoGroup();
    }

