function duplicateCompAndPrecomps(comp, duplicatedComps) {
    // Vérifier si cette comp a déjà été dupliquée
    if (duplicatedComps[comp.id]) {
        // Retourner la comp dupliquée existante
        return duplicatedComps[comp.id];
    }

    // Créer une nouvelle composition avec les mêmes paramètres
    var newComp = app.project.items.addComp(
        comp.name + "_dupliqué",
        comp.width,
        comp.height,
        comp.pixelAspect,
        comp.duration,
        comp.frameRate
    );

    // Stocker la comp dupliquée pour référence future
    duplicatedComps[comp.id] = newComp;

    // Parcourir les calques de la composition originale
    for (var i = 1; i <= comp.numLayers; i++) {
        var layer = comp.layer(i);
        var newLayer;

        if (layer.source) {
            var sourceItem = layer.source;

            if (sourceItem instanceof CompItem) {
                // Dupliquer la précomposition de manière récursive
                var duplicatedPrecomp = duplicateCompAndPrecomps(
                    sourceItem,
                    duplicatedComps
                );

                // Ajouter la précomp dupliquée en tant que calque
                newLayer = newComp.layers.add(duplicatedPrecomp);
            } else {
                // Source est un métrage ou autre élément
                newLayer = newComp.layers.add(sourceItem);
            }

            // Copier les propriétés du calque
            copyLayerProperties(layer, newLayer);
        } else {
            // Pour les calques sans source (texte, formes, solides, caméras, lumières, etc.)
            newLayer = layer.copyToComp(newComp);
        }
    }

    return newComp;
}

function copyLayerProperties(originalLayer, newLayer) {
    // Copier les propriétés de transformation
    copyPropertiesByMatchName(originalLayer.transform, newLayer.transform);

    // Copier le timing
    newLayer.inPoint = originalLayer.inPoint;
    newLayer.outPoint = originalLayer.outPoint;
    newLayer.startTime = originalLayer.startTime;
    newLayer.stretch = originalLayer.stretch;

    // Copier les modes de fusion et autres réglages généraux
    newLayer.blendingMode = originalLayer.blendingMode;
    newLayer.motionBlur = originalLayer.motionBlur;
    newLayer.adjustmentLayer = originalLayer.adjustmentLayer;
    newLayer.guideLayer = originalLayer.guideLayer;
    newLayer.threeDLayer = originalLayer.threeDLayer;

    // Copier les propriétés spécifiques aux calques de texte
    if (originalLayer instanceof TextLayer) {
        newLayer.threeDPerChar = originalLayer.threeDPerChar;
    }

    // Les effets et autres propriétés seront déjà copiés par copyToComp pour les calques sans source
}

function copyPropertiesByMatchName(sourceGroup, targetGroup) {
    for (var i = 1; i <= sourceGroup.numProperties; i++) {
        var sourceProperty = sourceGroup.property(i);
        var targetProperty = targetGroup.property(sourceProperty.matchName);

        if (
            targetProperty !== null &&
            !sourceProperty.isHidden &&
            !targetProperty.isHidden
        ) {
            if (sourceProperty.numProperties > 0) {
                // Propriété de groupe
                copyPropertiesByMatchName(sourceProperty, targetProperty);
            } else {
                try {
                    // Propriété individuelle
                    if (
                        sourceProperty.isModified &&
                        targetProperty.propertyValueType !==
                            PropertyValueType.NO_VALUE
                    ) {
                        targetProperty.setValue(sourceProperty.value);
                    }

                    // Copier les expressions
                    if (sourceProperty.expressionEnabled) {
                        targetProperty.expression = sourceProperty.expression;
                    }

                    // Copier les images clés
                    if (sourceProperty.numKeys > 0) {
                        targetProperty.setValuesAtTimes(
                            sourceProperty.keyTimes,
                            sourceProperty.keyValues
                        );
                    }
                } catch (e) {
                    // Ignorer les erreurs lors de la copie de cette propriété
                }
            }
        }
    }
}

// Fonction principale
function executeDuplicate() {
    var activeComp = app.project.activeItem;
    if (!(activeComp && activeComp instanceof CompItem)) {
        alert("Veuillez sélectionner une composition active.");
        return;
    }

    if (app.project.selection.length == 0) {
        alert(
            "Veuillez sélectionner une composition à dupliquer dans le panneau Projet."
        );
        return;
    }

    var selectedComp = null;

    // Trouver la première composition sélectionnée
    for (var i = 0; i < app.project.selection.length; i++) {
        if (app.project.selection[i] instanceof CompItem) {
            selectedComp = app.project.selection[i];
            break;
        }
    }

    if (!selectedComp) {
        alert("Veuillez sélectionner une composition valide à dupliquer.");
        return;
    }

    app.beginUndoGroup("Dupliquer Composition et Précompositions");

    var duplicatedComps = {};

    var newComp = duplicateCompAndPrecomps(selectedComp, duplicatedComps);

    // Vérifier si la composition active est différente de la composition sélectionnée
    if (activeComp !== selectedComp) {
        // Ajouter la nouvelle composition en tant que calque dans la composition active
        activeComp.layers.add(newComp);
    }

    app.endUndoGroup();
}

//executeDuplicate();
